/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _JACOBIAN_H_
#define _JACOBIAN_H_

#include <stdio.h>
#include <iostream>
#include "membership.h"
#include "rule.h"
#include "system.h"
#include "messages.h"
#include "ext_math.h"
#include "utilities.h"

// Declarations ################################################################

class Jacobian
{
    int n; // System order
    double **J; // Jacobian matrix
public:
    int order() const; // Return the order of the system
    double read_J(int row, int column) const; // Read a value of the jacobian matrix
    int write_J(double value, int row, int column); // Write a value in the jacobian matrix
    int resolv(System S, System C, const double * const x); // Computes the exact closed-loop jacobian in 'x' with Plant 'S' and Controller 'C'
    int aprox(System S, System C, const double * const x, double h=0.001); // Computes the approximation of the closed-loop jacobian in 'x' with the plant "S" and the controller "C" using finite differences.
    Jacobian(int n);
    Jacobian(System S, System C, const double * const x);
    ~Jacobian();
};

#endif
