/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "membership.h"

// Implementaciones ############################################################

// Membership  -Parent Classe- #################################################

double Membership::eval(double x) const {return 0.0;}

double Membership::evalder(double x) const {return 0.0;}

int Membership::test(void) const {return 1;}

int Membership::n_param(void) const {return n;}

TYPE_MF Membership::type(void) const {return type_mf;}

int Membership::type(TYPE_MF type_mf)
{
    if (type_mf!=this->type_mf)
    {
        if (type_mf>ZMF || type_mf<NOMF) // The fisrt and last value of TYPE_MF
            return 0;
        if (param!=NULL)
            delete [] param;
        n=MF_PARAM_NUMBER[type_mf];
        this->type_mf=type_mf;
        param=new double[n];
    }
    return 1;
}

int Membership::edit(int index,double value)
{
    if (index>=n)
        return 0;
    param[index]=value;
    return 1;
}

double Membership::read(int index) const
{
    if (index>=n)
    {
        #ifndef MATLAB_MEX_FILE
            std::cout<<E_ArrayExceded<<std::endl;
        #endif
        #ifdef MATLAB_MEX_FILE
            mexPrintf("%s\n",E_ArrayExceded);
        #endif
        return 0.0;
    }
    return param[index];
}

Membership & Membership::operator=(const Membership &P)
{
    int i;
    if (this==&P)
        return *this;
    if (param!=NULL)
        delete [] param;
    n=P.n_param();
    type_mf=P.type();
    param=new double[n];
    for (i=0;i<n;i++)
        param[i]=P.read(i);
    return *this;
}

Membership::Membership(const Membership &P)
{
    int i;
    n=P.n_param();
    type_mf=P.type();
    param=new double[n];
    for (i=0;i<n;i++)
        param[i]=P.read(i);
}

Membership::Membership()
{
    n=MF_PARAM_NUMBER[NOMF];
    type_mf=NOMF;
    param=NULL;
}

Membership::~Membership()
{
    if (param!=NULL)
    {
        delete [] param;
        param=NULL;
    }
}

// Nomf ########################################################################

double Nomf::eval(double x) const
{
    #ifndef MATLAB_MEX_FILE
        std::cout<<E_NomfUse<<std::endl;
    #endif
    #ifdef MATLAB_MEX_FILE
        mexErrMsgTxt(E_NomfUse);
    #endif
    return 0.0;
}

double Nomf::evalder(double x) const {return 0.0;}

Nomf::Nomf(void)
{
    n=MF_PARAM_NUMBER[NOMF];
    type_mf=NOMF;
    param=NULL;
}

// Bellmf ######################################################################

double Bellmf::eval(double x) const
{
    return 1/(1+pow(fabs((x-param[2])/param[0]),2*param[1]));
}

double Bellmf::evalder(double x) const
{
    return -2*param[1]*eval(x)*eval(x)*pow(fabs((x-param[2])/param[0]),(2*param[1]-1))*sign((x-param[2])/param[0])/param[0];
}

int Bellmf::test(void) const
{
    if (param[0]==0)
        return 0;
    return 1;
}

Bellmf::Bellmf(double a, double b, double c)
{
    n=MF_PARAM_NUMBER[BELLMF];
    type_mf=BELLMF;
    param=new double[n];
    param[0]=a;
    param[1]=b;
    param[2]=c;
}

// Constmf ####################################################################

double Constmf::eval(double x) const {return param[0];}

double Constmf::evalder(double x) const {return 0.0;}

Constmf::Constmf(double C)
{
    n=MF_PARAM_NUMBER[CONSTMF];
    type_mf=CONSTMF;
    param=new double[n];
    param[0]=C;
}

// Gaussmf #####################################################################

double Gaussmf::eval(double x) const
{
    double temp=(x-param[0])/param[1];
    return exp(-temp*temp);
}

double Gaussmf::evalder(double x) const
{
    return -2*eval(x)*(x-param[0])/(param[1]*param[1]);
}

int Gaussmf::test(void) const
{
    if (param[1]==0)
        return 0;
    if (param[1]<0)
        param[1]=-param[1];
    return 1;
}

Gaussmf::Gaussmf(double Center, double Beta)
{
    n=MF_PARAM_NUMBER[GAUSSMF];;
    type_mf=GAUSSMF;
    param=new double[n];
    param[0]=Center;
    param[1]=Beta;
}

// Gauss2mf ####################################################################

double Gauss2mf::eval(double x) const
{
    double y1,y2;
    if (x<param[0])
    {
        y1=(x-param[0])/param[1];
        y1=exp(-y1*y1);
    }
    else
        y1=1;
    if (x>param[2])
    {
        y2=(x-param[2])/param[3];
        y2=exp(-y2*y2);
    }
    else
        y2=1;
    return y1*y2;
}

double Gauss2mf::evalder(double x) const
{
    double d1=0.0,d2=0.0,temp;
    if (x<param[0] && x<param[2])
    {
        d1=2*this->eval(x)*(param[0]-x)/(param[1]*param[1]);
    }
    else if (x>param[0] && x>param[2])
    {
        d2=2*this->eval(x)*(param[2]-x)/(param[3]*param[3]);
    }
    else
    {
        temp=-(x-param[0])*(x-param[0])/(param[1]*param[1]);
        d1=exp(temp)*2*this->eval(x)*(param[0]-x)/(param[1]*param[1]);
        temp=-(x-param[2])*(x-param[2])/(param[3]*param[3]);
        d2=exp(temp)*2*this->eval(x)*(param[2]-x)/(param[3]*param[3]);
    }
    return d1+d2;
}

int Gauss2mf::test(void) const
{
    if (param[1]==0 || param[3]==0)
        return 0;
    if (param[1]<0)
        param[1]=-param[1];
    if (param[3]<0)
        param[3]=-param[3];
    return 1;
}

Gauss2mf::Gauss2mf(double Center1, double Beta1, double Center2, double Beta2)
{
    n=MF_PARAM_NUMBER[GAUSS2MF];
    type_mf=GAUSS2MF;
    param=new double[n];
    param[0]=Center1;
    param[1]=Beta1;
    param[2]=Center2;
    param[3]=Beta2;
}

// Pimf ########################################################################

double Pimf::eval(double x) const
{
    double s,z;
    if (param[0]>=param[1])
        s=(x>=(param[0]+param[1])/2);
    else if (x<=param[0])
        s=0;
    else if (x>param[0] && x<=((param[0]+param[1])/2))
        s=2*pow(((x-param[0])/(param[1]-param[0])),2);
    else if (x>=param[1])
        s=1;
    else
        s=1-2*pow(((param[1]-x)/(param[1]-param[0])),2);
    if (param[2]>=param[3])
        z=(x<=(param[2]+param[3])/2);
    else if (x<=param[2])
        z=1;
    else if (x>=param[3])
        z=0;
    else if (x>param[2] && x<=((param[2]+param[3])/2))
        z=1-2*pow((x-param[2])/(param[2]-param[3]),2);
    else
        z=2*pow((param[3]-x)/(param[2]-param[3]),2);
    return s*z;
}

double Pimf::evalder(double x) const
{
    double s,z,ds,dz;
    if (param[0]>=param[1])
    {
        s=(x>=(param[0]+param[1])/2);
	ds=0.0;
    }
    else if (x<=param[0])
    {
        s=0;
	ds=0.0;
    }
    else if (x>param[0] && x<=((param[0]+param[1])/2))
    {
        s=2*pow(((x-param[0])/(param[1]-param[0])),2);
	ds=4*(x-param[0])/pow(param[1]-param[0],2);
    }
    else if (x>=param[1])
    {
        s=1;
	ds=0.0;
    }
    else
    {
        s=1-2*pow(((param[1]-x)/(param[1]-param[0])),2);
	ds=4*(param[1]-x)/pow(param[1]-param[0],2);
    }
    if (param[2]>=param[3])
    {
        z=(x<=(param[2]+param[3])/2);
	dz=0.0;
    }
    else if (x<=param[2])
    {
        z=1;
	dz=0.0;
    }
    else if (x>=param[3])
    {
        z=0;
	dz=0.0;
    }
    else if (x>param[2] && x<=((param[2]+param[3])/2))
    {
        z=1-2*pow((x-param[2])/(param[2]-param[3]),2);
	dz=-4*(x-param[0])/pow(param[0]-param[1],2);
    }
    else
    {
        z=2*pow((param[3]-x)/(param[2]-param[3]),2);
	dz=-4*(param[1]-x)/pow(param[0]-param[1],2);
    }
    return (ds*z+dz*s);
}

Pimf::Pimf(double a, double b, double c, double d)
{
    n=MF_PARAM_NUMBER[PIMF];
    type_mf=PIMF;
    param=new double[n];
    param[0]=a;
    param[1]=b;
    param[2]=c;
    param[3]=d;
}

// PSigmf ################################################################

double PSigmf::eval(double x) const
{
    double s1=1/(1+exp(param[0]*(param[1]-x)));
    double s2=1/(1+exp(param[2]*(param[3]-x)));
    return s1*s2;
}

double PSigmf::evalder(double x) const
{
    double s1=1/(1+exp(param[0]*(param[1]-x)));
    double s2=1/(1+exp(param[2]*(param[3]-x)));
    double ds1=param[0]*eval(x)*eval(x)*exp(param[0]*(param[1]-x));
    double ds2=param[2]*eval(x)*eval(x)*exp(param[2]*(param[3]-x));
    return ds1*s2+ds2*s1;
}

PSigmf::PSigmf(double a1, double c1, double a2, double c2)
{
    n=MF_PARAM_NUMBER[PSIGMF];
    type_mf=PSIGMF;
    param=new double[n];
    param[0]=a1;
    param[1]=c1;
    param[2]=a2;
    param[3]=c2;
}

// Smf #########################################################################

double Smf::eval(double x) const
{
    if (param[0]>=param[1])
        return (x>=(param[0]+param[1])/2);
    else if (x<=param[0])
        return 0;
    else if (x>param[0] && x<=((param[0]+param[1])/2))
        return 2*pow(((x-param[0])/(param[1]-param[0])),2);
    else if (x>=param[1])
        return 1;
    else
        return 1-2*pow(((param[1]-x)/(param[1]-param[0])),2);
}

double Smf::evalder(double x) const
{
    if (x<=param[0] || x>=param[1])
        return 0;
    else if (x>param[0] && x<=((param[0]+param[1])/2))
        return 4*(x-param[0])/pow(param[1]-param[0],2);
    else
        return 4*(param[1]-x)/pow(param[1]-param[0],2);
}

Smf::Smf(double a, double b)
{
    n=MF_PARAM_NUMBER[SMF];
    type_mf=SMF;
    param=new double[n];
    param[0]=a;
    param[1]=b;
}

// Sigmf #################################################################

double Sigmf::eval(double x) const
{
    return 1/(1+exp(param[0]*(param[1]-x)));
}

double Sigmf::evalder(double x) const
{
    return param[0]*eval(x)*eval(x)*exp(param[0]*(param[1]-x));
}

Sigmf::Sigmf(double a, double c)
{
    n=MF_PARAM_NUMBER[SIGMF];
    type_mf=SIGMF;
    param=new double[n];
    param[0]=a;
    param[1]=c;
}

// Sig2mf ################################################################

double Sig2mf::eval(double x) const
{
    double s1=1/(1+exp(param[0]*(param[1]-x)));
    double s2=1/(1+exp(param[2]*(param[3]-x)));
    return s1-s2;
}

double Sig2mf::evalder(double x) const
{
    double ds1=param[0]*eval(x)*eval(x)*exp(param[0]*(param[1]-x));
    double ds2=param[2]*eval(x)*eval(x)*exp(param[2]*(param[3]-x));
    return ds1-ds2;
}

Sig2mf::Sig2mf(double a1, double c1, double a2, double c2)
{
    n=MF_PARAM_NUMBER[SIG2MF];
    type_mf=SIG2MF;
    param=new double[n];
    param[0]=a1;
    param[1]=c1;
    param[2]=a2;
    param[3]=c2;
}

// TRAPMF ###############################################################

double Trapmf::eval(double x) const
{
    if (x>param[0] && x<param[1])
        return (x-param[0])/(param[1]-param[0]);
    else if (x>param[2] && x<param[3])
        return (param[3]-x)/(param[3]-param[2]);
    else if (x>=param[1] && x<=param[2])
        return 1;
    else
        return 0;
}

double Trapmf::evalder(double x) const
{ // On singular point df(x) = [df(x+) + dx(x-)] / 2
    if (x>param[0] && x<param[1])
        return 1/(param[1]-param[0]);
    else if (x>param[2] && x<param[3])
        return 1/(param[3]-param[2]);
    else if (x==param[0] || x==param[1])
        return 0.5/(param[1]-param[0]);
    else if (x==param[2] || x==param[3])
        return 0.5/(param[3]-param[2]);
    else
        return 0;
}

int Trapmf::test(void) const
{
    if (param[2]>=param[3])
        param[2]=param[3]-M_EPS;
    if (param[1]>=param[2])
        param[1]==param[2]-M_EPS;
    if (param[0]>=param[1])
        param[0]=param[1]-M_EPS;
    return 1;
}

Trapmf::Trapmf(double a, double b, double c, double d)
{
    n=MF_PARAM_NUMBER[TRAPMF];
    type_mf=TRAPMF;
    param=new double[n];
    param[0]=a;
    param[1]=b;
    param[2]=c;
    param[3]=d;
}

// Trap2mf #####################################################################

double Trap2mf::eval(double x) const
{
	if (x<=param[0] || x>=param[7] || (x>=param[3] && x<=param[4]))
		return 0;
    else if (x>param[0] && x<param[1])
        return (x-param[0])/(param[1]-param[0]+M_EPS);
    else if (x>param[2] && x<param[3])
        return (param[3]-x)/(param[3]-param[2]+M_EPS);
    else if (x>param[4] && x<param[5])
        return (x-param[4])/(param[5]-param[4]+M_EPS);
    else if (x>param[6] && x<param[7])
        return (param[7]-x)/(param[7]-param[6]+M_EPS);
    else
        return 1;
}

double Trap2mf::evalder(double x) const
{// On singular point df(x) = [df(x+) + dx(x-)] / 2
    if (x>param[0] && x<param[1])
        return 1/(param[1]-param[0]+M_EPS);
    else if (x>param[2] && x<param[3])
        return 1/(param[3]-param[2]+M_EPS);
    else if (x>param[4] && x<param[5])
        return 1/(param[5]-param[4]+M_EPS);
    else if (x>param[6] && x<param[7])
        return 1/(param[7]-param[6]+M_EPS);
    else if (x==param[0] || x==param[1])
        return 0.5/(param[1]-param[0]+M_EPS);
    else if (x==param[2] || x==param[3])
        return 0.5/(param[3]-param[2]+M_EPS);
    else if (x==param[4] || x==param[5])
        return 0.5/(param[5]-param[4]+M_EPS);
    else if (x==param[6] || x==param[7])
        return 0.5/(param[7]-param[6]+M_EPS);
    else
        return 0;
}

int Trap2mf::test(void) const
{
	for (int i=0;i<n-1;i++)
	{
		if (param[i]>=param[i+1])
			param[i]=param[i+1]-M_EPS;
	}
    return 1;
}

Trap2mf::Trap2mf(double a1, double b1, double c1, double d1,double a2, double b2, double c2, double d2)
{
    n=MF_PARAM_NUMBER[TRAP2MF];
    type_mf=TRAP2MF;
    param=new double[n];
    param[0]=a1;
    param[1]=b1;
    param[2]=c1;
    param[3]=d1;
    param[4]=a2;
    param[5]=b2;
    param[6]=c2;
    param[7]=d2;
}

// Trimf #######################################################################

double Trimf::eval(double x) const
{
    if (x>param[0] && x<=param[1])
        return (x-param[0])/(param[1]-param[0]);
    else if (x>param[1] && x<param[2])
        return (param[2]-x)/(param[2]-param[1]);
    else
        return 0;
}

double Trimf::evalder(double x) const
{// On singular point df(x) = [df(x+) + dx(x-)] / 2
    if (x>param[0] && x<param[1])
        return 1/(param[1]-param[0]);
    else if (x>param[1] && x<param[2])
        return 1/(param[2]-param[1]);
    else if (x==param[0])
        return 0.5/(param[1]-param[0]);
    else if (x==param[1])
        return (1/(param[1]-param[0])+1/(param[2]-param[1]))/2;
    else if (x==param[2])
        return 0.5/(param[2]-param[1]);
    else
        return 0;
}

int Trimf::test(void) const
{
    if (param[1]>=param[2])
        param[1]==param[2]-M_EPS;
    if (param[0]>=param[1])
        param[0]=param[1]-M_EPS;
    return 1;
}

Trimf::Trimf(double a, double b, double c)
{
    n=MF_PARAM_NUMBER[TRIMF];
    type_mf=TRIMF;
    param=new double[n];
    param[0]=a;
    param[1]=b;
    param[2]=c;
}

// Zmf #########################################################################

double Zmf::eval(double x) const
{
    if (param[0]>=param[1])
        return (x<=(param[0]+param[1])/2);
    else if (x<=param[0])
        return 1;
    else if (x>=param[1])
        return 0;
    else if (x>param[0] && x<=((param[0]+param[1])/2))
        return 1-2*pow((x-param[0])/(param[0]-param[1]),2);
    else
        return 2*pow((param[1]-x)/(param[0]-param[1]),2);
}

double Zmf::evalder(double x) const
{
    if (x<=param[0] || x>=param[1])
        return 0;
    else if (x>param[0] && x<=((param[0]+param[1])/2))
        return -4*(x-param[0])/pow(param[0]-param[1],2);
    else
        return -4*(param[1]-x)/pow(param[0]-param[1],2);
}

Zmf::Zmf(double a, double b)
{
    n=MF_PARAM_NUMBER[ZMF];
    type_mf=ZMF;
    param=new double[n];
    param[0]=a;
    param[1]=b;
}

// Virtual Constructor #########################################################

Membership * createMF(TYPE_MF t)
{
    switch(t)
    {
        case NOMF:
            return new Nomf;
        case BELLMF:
            return new Bellmf;
        case CONSTMF:
        	return new Constmf;
        case GAUSSMF:
            return new Gaussmf;
        case GAUSS2MF:
            return new Gauss2mf;
        case PIMF:
            return new Pimf;
        case PSIGMF:
            return new PSigmf;
        case SMF:
            return new Smf;
        case SIGMF:
            return new Sigmf;
        case SIG2MF:
            return new Sig2mf;
        case TRAPMF:
            return new Trapmf;
        case TRAP2MF:
            return new Trap2mf;
        case TRIMF:
            return new Trimf;
        case ZMF:
            return new Zmf;
        default:
            return NULL;
    }
}
