/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MESSAGES_H_
#define _MESSAGES_H_

// Para evitar que MATLAB se cierre en Linux con mexErrMsgTxt se redefine
#ifndef strcmpi
    #define ERRORMSG(men) {mexPrintf("??? Error using ==> %s\b\b\b\n%s\n\n",__FILE__,men);return;}
#else
    #define ERRORMSG(men) {mexErrMsgTxt(men);}
#endif

// Definición de Constantes empleadas en el Programa
#define MAX_CHAR_LONG 256  // Maximum length of strings
#define MAX_FILE_NAME 256  // Maximum length of file names
#define PRECISION 10      // Default number of digits when writing to file or screen

// Error Messages( E_... ) #################################################
    // Model
        const static char *E_Model={"Error reading model."};
        const static char *E_NoSugeno={"Model 'type' should be 'sugeno'."};
        const static char *E_NoOutputs={"Error, the model has not any Output."};
        const static char *E_NoInputs={"Error, the model has not any Input."};
        const static char *E_NoRules={"Error, some Outputs of the model have not any Rule."};
        const static char *E_No1Conseq={"Only one consequent for rule is allowed."};
        const static char *E_SameIns={"All models must have the same inputs."};
        const static char *E_NoCoherent={"The Controller is not coherent with the Plant."};
        const static char *E_InNoCoherent={"Inputs arguments are not coherent with model inputs."};
        const static char *E_BadModel={"Undefined or wrong fuzzy model.\nCheck rules, membership functions and theirs parameters"};
        const static char *E_NoCoherent_BadX={"The Controller is not coherent with the Plant, or the point is not coherent with the closed loop system."};
        const static char *E_ArgNoValid={"Empty and NaN matrices are not valid inputs arguments."};
        const static char *E_AccuracyNoNum={"The accuracy should be a number."};
        const static char *E_Acquire={"Error reading model or incorrect initial model."};
        const static char *E_Store={"Error storing data. Wrong vector size or any data not valid for membership function, i.e. a zero width in Gaussmf function."};
    // FIS
        const static char *E_NoFIS={"Fuzzy Model must be a FIS structure."};
        const static char *E_CreateFIS={"Error at create FIS."};
        const static char *E_NoProd={"Implication method must be 'prod'."};
        const static char *E_FISOut={"Error writing FIS."};
        const static char *E_RulesExceded={"Rule limit exceeded."};
		const static char *E_InputExceded={"Input limit exceeded."};
        const static char *E_OutputExceded={"Output limit exceeded."};
    // Membership Functions
        const static char *E_MFType={"Error reading membership function type:"};
        const static char *E_BadMF={"Membership function was not identified."};
        const static char *E_MFTypeDef={"Error in membership function type."};
        const static char *E_ParamMF={"Error reading membership function parameters."};
        const static char *E_NoValidFP={"Any parameter of membership function is not valid."};
        const static char *E_MFDef={"Error in membership function definition."};
        const static char *E_NomfUse={"NOMF function not must be evaluated. It is possible that there is an error in the souorces."};
    // Consequent
        const static char *E_Conseq={"It is not posible read consequent type in rule:"};
        const static char *E_ParamConseq={"Error reading TSK's consequent parameters."};
        const static char *E_BadConseq={"Not valid cosequent."};
    // Files
        const static char *E_FileInput={"Input file error."};
        const static char *E_FileOutput={"Output file error."};
    // Others
        const static char *E_ArrayExceded={"Array limit exceded."};
        const static char *E_ChangingParameter={"Error changing the parameter."};
        const static char *E_WritingRule={"Error writing the rule"};
        const static char *E_NoSumProm={"Error, 'defuzzMethod' should be 'wtaver'."};
        const static char *E_NumberParam={"Number of parameters incorrect."};
        const static char *E_NumberArg={"Error in number of arguments."};
        const static char *E_NumberArgIn={"Error in inputs arguments"};
        const static char *E_NumberArgOut={"Error in outputs arguments."};
        const static char *E_Column={"The state vector must be a column vector."};
        const static char *E_File={"Rules and Outputs should be a file vector."};
        const static char *E_PointCoherent={"The point/s should be coherent with fuzzy models."};
        const static char *E_InNames={"Error reading names of inputs."};
        const static char *E_OutNames={"Error reading names of outputs."};
        const static char *E_R_No_Scalar={"The number of rule must be a scalar."};
        const static char *E_Out_No_Scalar={"The number of output must be a scalar."};
        const static char *E_Epoch_No_Scalar={"The number of epoch must be a scalar."};
        const static char *E_Alfa_No_Scalar={"Alfa must be a scalar."};
        const static char *E_Error_No_Scalar={"The error must be a scalar."};
        const static char *E_ModelError={"Modeling error."};
        const static char *E_H_GE_0={"'h' must be >= 0."};


// Mensajes de Usuario ( U_... ) ###############################################
    // FIS
        const static char *U_FISName={"Unnamed"};
        const static char *U_FISInput={"Input"};
        const static char *U_FISOutput={"Output"};
        const static char *U_RuleAbbreviation={"R"};
        const static char *U_InputAbbreviation={"-In"};
        const static char *U_OutputAbreviation={"-Out"};
        const static char *U_If={"IF"};
        const static char *U_Is={"is"};
        const static char *U_And={"and"};
        const static char *U_OR={"or"};
        const static char *U_Then={"THEN"};
    // Otros
        const static char *U_SeeManual={"See also Fuzzy Logic Toolbox Manual."};
        const static char *U_SeeHelp={"see the help."};
        const static char *U_CheckModel={"Check model's definition."};
        const static char *U_MathException={"Warning. Math exception generated."};
        const static char *U_Overflow={"Happened an overflow."};
        const static char *U_FewData={"There are few data for modeling the system."};

// Otros Mensajes ( O_... ) ####################################################
    // Entradas
        const static char *O_Input={"Input:"};
        const static char *O_OfIn={"of in"};
    // Salidas
        const static char *O_OfOut={"of out"};
    // Reglas
        const static char *O_RuleWeigh={"The rule's weigh:"};
        const static char *O_RuleConnection={"The rule's connection:"};
        const static char *O_AntecedentSize={"The size of rule's antecedent"};
        const static char *O_ConsequentSize={"The size of rule's consequent"};
        const static char *O_Rule={"Rule:"};
    //Otros
        const static char *O_DifferentOf={"is different of"};
        const static char *O_IsNotCorrect={"is not correct."};
        const static char *O_MF={"Membership Function:"};
        const static char *O_OfPlant={"of the Plant"};
        const static char *O_OfController={"of Controller"};

#endif
