/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rule.h"

// Implementations ############################################################

void Rule::clean(void)
{
    int v;
    if (membership!=NULL && in>0)
    {
        for (v=0;v<in;v++)
        {
            if (membership[v]!=NULL)
            {
                delete membership[v];
                membership[v]=NULL;
            }
        }
        delete [] membership;
    }
    membership=NULL;
    if (TSK!=NULL)
    {
        delete [] TSK;
        TSK=NULL;
    }
}

int Rule::n_inputs(void) const {return in;}

int Rule::test(void) const
{
    int v;
    if (membership==NULL || in==0)
        return 0;
    else
    {
        for (v=0;v<in;v++)
        {
            if (membership[v]==NULL)
                return 0;
            if (membership[v]->test()==0)
                return 0;
        }
    }
    return 1;
}

int Rule::write_TSK(double value, int input)
{
    if (input>in)
        return 0;
    TSK[input]=value;
    return 1;
}

double Rule::read_TSK(int input) const
{
    if (input>in)
    {
        #ifndef MATLAB_MEX_FILE
            std::cout<<E_ArrayExceded<<std::endl;
        #endif
        #ifdef MATLAB_MEX_FILE
            mexPrintf("%s\n",E_ArrayExceded);
        #endif
        return 0;
    }
    return TSK[input];
}

int Rule::read_function(Membership &P, int input) const
{
    if (input>=in)
        return 0;
    P=*membership[input];
    return 1;
}

Membership *Rule::read_function(int input)
{
    if (input>=in)
    {
        #ifndef MATLAB_MEX_FILE
            std::cout<<E_ArrayExceded<<std::endl;
        #endif
        #ifdef MATLAB_MEX_FILE
            mexPrintf("%s\n",E_ArrayExceded);
        #endif
        return NULL;
    }
    return membership[input];
}

int Rule::write_function(const Membership &P, int input)
{
    if (input>=in)
        return 0;
    *(membership[input])=P;
    return 1;
}

int Rule::change_function(TYPE_MF type,int input)
{
    if (input>=in)
        return 0;
    if (membership[input]!=NULL)
        delete membership[input];
    membership[input]=createMF(type);
    return 1;
}

void Rule::initialize(int in)
{
    int v;
    if (this->in!=0)
        clean();
    this->in=in;
    TSK=new double [(in+1)];
    membership=new Membership *[in];
    for (v=0;v<in;v++)
        membership[v]=NULL;
}

Rule& Rule::operator=(const Rule &R)
{
    int v;
    if(this==&R)
        return *this;
    initialize(R.n_inputs());
    for (v=0;v<in;v++)
    {
        membership[v]=createMF(R.membership[v]->type());
        (*membership[v])=(*R.membership[v]);
    }
    for (v=0;v<=in;v++)
        TSK[v]=R.read_TSK(v);
    return *this;
}

Rule::Rule()
{
    TSK=NULL;
    membership=NULL;
    in=0;
}

Rule::Rule(int in)
{
    this->in=0;
    initialize(in);
}

Rule::Rule(const Rule &R)
{
    int v;
    in=0;
    initialize(R.n_inputs());
    for (v=0;v<in;v++)
    {
        membership[v]=createMF(R.membership[v]->type());
        (*membership[v])=(*R.membership[v]);
    }
    for (v=0;v<=in;v++)
        TSK[v]=R.read_TSK(v);
}

Rule::~Rule(){clean();}
