/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "utilities.h"

// TXT2System #################################################################

int TXT2System(const char *file, System &S)
{
    double d;
    int i,j,l,t,v,in,out,Err;
    char type[MAX_SIZE_TYPE_NAME];
    TYPE_MF T;
    Membership *P;
    Rule *R;
    FILE *F=fopen(file,"rt");
    if (!F)
        return 0;
    Err=fscanf(F,"%d",&in);
    Err&=fscanf(F,"%d",&out);
    if (!Err || out<1 || in<1)
    {
        fclose(F);
        return 0;
    }

    int *r=new int[out];
    for (j=0;j<out;j++)
    {
        Err=fscanf(F,"%d",r+j);
        if (!Err || *(r+j)<1)
        {
            delete [] r;
            fclose(F);
            return 0;
        }
    }
    System temp(in,out,r);

    // Get system data ########################################################
    for (j=0;j<out;j++)
    {
        for (l=0;l<r[j];l++)
        {
            R=temp.read_Rule(l,j);
            for (i=0;i<in;i++)
            {
                Err&=fscanf(F,"%s",&(type[0]));
                for (int iT=0;iT<=(int)ZMF;iT++)
                {
                	T=(TYPE_MF)iT;
                    if (!strcmpi(MF_NAMES[T],type))
                        break;
                }
                if (T<NOMF || T>ZMF)
                {
                    delete [] r;
                    fclose(F);
                    return 0;
                }
                if (!R->change_function(T,i))
                {
                    delete [] r;
                    fclose(F);
                    return 0;
                }
                P=createMF(T);
                for (v=0;v<MF_PARAM_NUMBER[(int)T];v++)
                {
                    Err&=fscanf(F,"%lf",&d);
                    if(!P->edit(v,d))
                    {
                        delete [] r;
                        fclose(F);
                        delete P;
                        return 0;
                    }
                }
                if (P->test()==0)
                {
                    delete [] r;
                    fclose(F);
                    delete P;
                    return 0;
                }
                if(!R->write_function(*P,i))
                {
                    delete [] r;
                    fclose(F);
                    delete P;
                    return 0;
                }
                delete P;
            }
            for (i=0;i<=in;i++)
            {
                Err&=fscanf(F,"%lf",&d);
                if(!R->write_TSK(d,i))
                {
                    delete [] r;
                    fclose(F);
                    return 0;
                }
            }
        }
    }
    delete [] r;

	double *limits_min_in=new double[in];
	double *limits_max_in=new double[in];
	for (i=0;i<in;i++)
	{
		Err&=fscanf(F,"%lf",limits_min_in+i);
		Err&=fscanf(F,"%lf",limits_max_in+i);
	}
	temp.in_min(limits_min_in);
	temp.in_max(limits_max_in);
	delete [] limits_min_in;
	delete [] limits_max_in;

	double *limits_min_out=new double[out];
	double *limits_max_out=new double[out];
	for (j=0;j<out;j++)
	{
		Err&=fscanf(F,"%lf",limits_min_out+j);
		Err&=fscanf(F,"%lf",limits_max_out+j);
	}
	temp.out_min(limits_min_out);
	temp.out_max(limits_max_out);
	delete [] limits_min_out;
	delete [] limits_max_out;

    fclose(F);
    if(!Err)
        return 0;
    S=temp;
    return 1;
}

// System2TXT #################################################################

int System2TXT(System &S, const char *file)
{
    int i,p,l,j,n,m;
    if (S.test()==0)
        return 0;
    FILE *Fs=fopen(file,"wt");
    if (!Fs)
        return 0;
    Rule *R;
    Membership *P;
    char type[MAX_CHAR_LONG];
    n=S.inputs();
    m=S.outputs();
    fprintf(Fs,"%d\t%d\n",n,m);
    for (j=0;j<m;j++)
        fprintf(Fs,"%d\t",S.rules(j));
    fprintf(Fs,"\n\n");
    for (j=0;j<m;j++)
    { // Output
        for (l=0;l<S.rules(j);l++)
        {// Rule
            R=S.read_Rule(l,j);
            for (i=0;i<n;i++)
            {// Antecedent of the rule
                P=R->read_function(i);
                if (P==NULL)
                {
                    fclose(Fs);
                    return 0;
                }
                fprintf(Fs,"%s\t",MF_NAMES[(int)P->type()]);
                for (p=0;p<P->n_param();p++)
                    fprintf(Fs,"%.10g\t",P->read(p));
            }
            fprintf(Fs,"\n");
            for (i=0;i<=n;i++)// Consequent of the rule
                fprintf(Fs,"%.10g\t",R->read_TSK(i));
            fprintf(Fs,"\n\n");
        }
    }
	for (i=0;i<n;i++)
		fprintf(Fs,"%.10g\t%.10g\n",S.in_min(i),S.in_max(i));
	fprintf(Fs,"\n");
	for (j=0;j<m;j++)
		fprintf(Fs,"%.10g\t%.10g\n",S.out_min(j),S.out_max(j));
	fprintf(Fs,"\n");
    fclose(Fs);
    return 1;
}

// Evaluate ###################################################################

int Evaluate(const double * const x, double *output, System &S)
{
    int i,j,k,l,M_max=0; // Indices

    const int n=S.outputs(); // Order
    const int m=S.inputs()-n; // Number of control inputs
    int *M=new int[n]; // Number of rules for each output of the system
    for (i=0;i<n;i++)
    {
        M[i]=S.rules(i);
        if (M[i]>M_max)
            M_max=M[i];
    }

    double *X=new double [n+1];
    X[0]=1;
    for (i=0;i<n;i++)
        *(X+i+1)=*(x+i); // State vector extension

    // Auxiliary values
    int toreturn=1;
    double temp, Sum_W, at, bt;
    double *W=new double [M_max]; // Matching degree of the plant
    Membership *P;
    Rule *R;

    for (i=0;i<n;i++)
    {
        Sum_W=0;
        for (l=0;l<M[i];l++)
        {
            R=S.read_Rule(l,i);
            W[l]=1;
            for (k=0;k<n;k++)
            {
                P=R->read_function(k);
				if (P->type()!=NOMF) // NOMF doesn't affect to the system
					W[l]=W[l]*P->eval(x[k]);
            }
            for (j=0;j<m;j++)
            {
                P=R->read_function(j+n);
				if (P->type()!=NOMF) // NOMF doesn't affect to the system
					W[l]=W[l]*P->eval(x[j+n]);
            }
            // W[l][i]
            Sum_W=Sum_W+W[l];
        }
        (*output)=0;
        for (k=0;k<=n;k++)
        {
            at=0;
            for (l=0;l<M[i];l++)
                at=at+W[l]*S.read_Rule(l,i)->read_TSK(k);
//            at=at/Sum_W; // at[k][i] // The division is made at the end
            (*output)=(*output)+at*X[k];
        }
        for (j=0;j<m;j++)
        {
            bt=0;
            for (l=0;l<M[i];l++)
                bt=bt+W[l]*S.read_Rule(l,i)->read_TSK(j+n+1);
//            bt=bt/Sum_W; // bt[j][i] // The division is made at the end
            (*output)=(*output)+bt*x[j+n]; // X is actually U here
        }
        (*output)=(*output)/(Sum_W+M_EPS);
        if (toreturn==1 && ((fabs(*output))>=(fabs(*output)+1)))
            toreturn=-1;
        output++;
    }

    // Free dynamic memory
    delete [] M;
    delete [] W;
    delete [] X;

    return toreturn;
}

int Evaluate(const double * const x, double *dX, System &S, System &C)
{

    int i,j,k,l,r,M_max=0,N_max=0; // Indices
    const int n=S.outputs(); // Order
    const int m=C.outputs(); // Number of control inputs

    int *M=new int[n]; // Number of rules for each output of the plant
    for (i=0;i<n;i++)
    {
        *(M+i)=S.rules(i);
        if (M[i]>M_max)
            M_max=M[i];
    }
    int *N=new int[m]; // Number of rules for each output of the controller
    for (j=0;j<m;j++)
    {
        *(N+j)=C.rules(j);
        if (N[j]>N_max)
            N_max=N[j];
    }

    // Check if the plant and the controller are coherent
    if (S.outputs()!=C.inputs())
    {
        delete [] M;
        delete [] N;
        return 0;
    }
    if (n!=S.inputs()-C.outputs())
    {
        delete [] M;
        delete [] N;
        return 0;
    }

    // State vector extension
    double *X=new double [n+1];
    X[0]=1;
    for (i=0;i<n;i++)
        *(X+i+1)=*(x+i);

    int toreturn=1;
    double temp, at, Sum_activ;
    double *Sum_W=new double[n];
    double **activ=new double *[N_max]; // Matching degree of the rules of the controller (omega)
    double **ct=new double *[n+1]; // Variable coefficient for the controller
    double *u=new double [m]; // Control signal
    double **W=new double *[M_max]; // Matching degree of the rules of the plant
    double *bt=new double [m]; // Variable coefficient 'b' for the plant
    Rule *R;
	Membership *P;

    for (l=0;l<M_max;l++)
        W[l]=new double[n];
    for (r=0;r<N_max;r++)
        activ[r]=new double[m];
    for (i=0;i<=n;i++)
        ct[i]=new double[m];
    // Calculation of dX 
    for (j=0;j<m;j++)
    {
        Sum_activ=0;
        for (r=0;r<N[j];r++)
        {
            R=C.read_Rule(r,j);
            activ[r][j]=1;
            for (k=0;k<n;k++)
			{
				P=R->read_function(k);
				if (P->type()!=NOMF) // NOMF doesn't affect to the system
					activ[r][j]*=P->eval(x[k]);
			}
            Sum_activ+=activ[r][j];
        } // activ[r][j]
        u[j]=0;
        for (k=0;k<=n;k++)
        {
            ct[k][j]=0;
            for (r=0;r<N[j];r++)
                ct[k][j]+=activ[r][j]*C.read_Rule(r,j)->read_TSK(k);
            ct[k][j]/=(Sum_activ+M_EPS); // ct[k][j]
            u[j]+=X[k]*ct[k][j];
        } // u[j]
    }

    for (i=0;i<n;i++)
    {
        Sum_W[i]=0;
        for (l=0;l<M[i];l++)
        {
            W[l][i]=1;
            R=S.read_Rule(l,i);
            for (k=0;k<n;k++)
			{
				P=R->read_function(k);
				if (P->type()!=NOMF) // NOMF doesn't affect to the system
					W[l][i]*=P->eval(x[k]);
			}
            for (j=0;j<m;j++)
			{
				P=R->read_function(j+n);
				if (P->type()!=NOMF) // NOMF doesn't affect to the system
					W[l][i]*=P->eval(u[j]);
			}
            // W[l][i]
            Sum_W[i]+=W[l][i];
        }
    }
    for (i=0;i<n;i++)
    {
        (*dX)=0;
        for (j=0;j<m;j++)
        {
            bt[j]=0;
            for (l=0;l<M[i];l++)
                bt[j]+=W[l][i]*S.read_Rule(l,i)->read_TSK(j+n+1);
//            bt[j]=bt[j]/Sum_W[i]; // bt[j][i] // The division is made at the end
        }
        for (k=0;k<=n;k++)
        {
            at=0;
            for (l=0;l<M[i];l++)
                at+=W[l][i]*S.read_Rule(l,i)->read_TSK(k);
//            at=at/Sum_W[i]; // at[k][i] // The division is made at the end
            // Feedback loop
            temp=0;
            for (j=0;j<m;j++)
                temp+=bt[j]*ct[k][j];
            (*dX)+=(at+temp)*X[k];
        }
        (*dX)/=(Sum_W[i]+M_EPS);
        if (toreturn==1 && (fabs(*dX)>=(fabs(*dX)+1)))
            toreturn=-1;
        dX++;
    }
    // Free the dynamic memory
    delete [] M;
    delete [] N;
    delete [] bt;
    for (i=0;i<=n;i++)
        delete [] ct[i];
    delete [] ct;
    for (r=0;r<N_max;r++)
        delete [] activ[r];
    delete [] activ;
    for (l=0;l<M_max;l++)
        delete [] W[l];
    delete [] W;
    delete [] u;
    delete [] Sum_W;
    delete [] X;

    return toreturn;
}

// linearization ##################################################################

int linearization(System &S, const double *const x, const double *const u, double **A, double **B, double *F)
{
    // Indices
    int i,k,q,j,t,l,p,M_max=0;
    int toreturn=1;
    const int n=S.outputs(); // Order
    const int m=S.inputs()-n; // Number of control inputs
    int *M=new int[n]; // Number of rules of the plant for each output
    for (i=0;i<n;i++)
    {
        *(M+i)=S.rules(i);
        if (M[i]>M_max)
            M_max=M[i];
    }

    double temp1,temp2;
    double a1,a2,b1,b2,at,bt;
    double *X=new double [n+1]; // State vector extension
    X[0]=1;
    for (i=0;i<n;i++)
        X[i+1]=x[i];
    double **W=new double *[M_max]; // Matching degree of the rules of the plant
    double ***der_wx=new double**[M_max]; // dW/dx
    double ***der_wu=new double**[M_max]; // dW/du
    for (l=0;l<M_max;l++)
    {
        W[l]=new double[n];
        der_wx[l]=new double*[n];
        der_wu[l]=new double*[n];
        for (i=0;i<n;i++)
        {
            der_wx[l][i]=new double[n];
            der_wu[l][i]=new double[m];
        }
    }
    double *Sum_W=new double[n];
    Membership *P;
    Rule *R;
    // Algorithm
    for (i=0;i<n;i++)
    {
        Sum_W[i]=0;
        for (l=0;l<M[i];l++)
        {
            R=S.read_Rule(l,i);
            W[l][i]=1;
            for (k=0;k<n;k++)
            {
                P=R->read_function(k);
				if (P->type()!=NOMF) // NOMF doesn't affect to the system
					W[l][i]=W[l][i]*P->eval(x[k]);
            }
            for (j=0;j<m;j++)
            {
                P=R->read_function(j+n);
				if (P->type()!=NOMF) // NOMF doesn't affect to the system
					W[l][i]=W[l][i]*P->eval(u[j]);
            }
            // W[l][i]
            Sum_W[i]=Sum_W[i]+W[l][i];
            for (q=0;q<n;q++)
            {
				P=R->read_function(q);
				if (P->type()!=NOMF) // NOMF doesn't affect to the system
				{
					der_wx[l][i][q]=P->evalder(x[q]);
					for (k=0;k<n;k++)
					{
						if (k!=q)
						{
							P=R->read_function(k);
							if (P->type()!=NOMF) // NOMF doesn't affect to the system
								der_wx[l][i][q] = der_wx[l][i][q] * P->eval(x[k]);
						}
					}
					for (j=0;j<m;j++)
					{
						P=R->read_function(j+n);
						if (P->type()!=NOMF) // NOMF doesn't affect to the system
							der_wx[l][i][q] = der_wx[l][i][q] * P->eval(u[j]);
					}
				}
				else
					der_wx[l][i][q]=0;
            } // der_wx[l][i][q]
            for (t=0;t<m;t++)
            {
				P=R->read_function(t+n);
				if (P->type()!=NOMF) // NOMF doesn't affect to the system
				{
					der_wu[l][i][t]=P->evalder(u[t]);
					for (k=0;k<n;k++)
					{
						P=R->read_function(k);
						if (P->type()!=NOMF) // NOMF doesn't affect to the system
							der_wu[l][i][t] = der_wu[l][i][t] * P->eval(x[k]);
					}
					for (j=0;j<m;j++)
					{
						if (j!=t)
						{
							P=R->read_function(j+n);
							if (P->type()!=NOMF) // NOMF doesn't affect to the system
								der_wu[l][i][t] = der_wu[l][i][t] * P->eval(u[j]);
						}
					}
				}
				else
					der_wu[l][i][t] = 0;
            }
        }// Sum_W[i]
    }
    for (i=0;i<n;i++)
    {
        for (t=0;t<m;t++) // To obtain B
        {
            bt=0;
            for (l=0;l<M[i];l++)
            {
                R=S.read_Rule(l,i);
                bt=bt+W[l][i]*R->read_TSK(t+n+1);
            }
            bt=bt/(Sum_W[i]+M_EPS); // bt[t][i]
            temp2=0;
            for (k=0;k<=n;k++)
            {
                temp1=0;
                for (l=0;l<M[i];l++)
                {
                    a1=S.read_Rule(l,i)->read_TSK(k);
                    for (p=0;p<M[i];p++)
                    {
                        a2=S.read_Rule(p,i)->read_TSK(k);
                        temp1=temp1+der_wu[l][i][t]*W[p][i]*(a1-a2);
                    }
                }
                temp1=temp1*X[k]/(Sum_W[i]*Sum_W[i]+M_EPS);
                temp2=temp2+temp1;
            }
            for (j=0;j<m;j++)
            {
                temp1=0;
                for (l=0;l<M[i];l++)
                {
                    b1=S.read_Rule(l,i)->read_TSK(j+n+1);
                    for (p=0;p<M[i];p++)
                    {
                        b2=S.read_Rule(p,i)->read_TSK(j+n+1);
                        temp1=temp1+der_wu[l][i][t]*W[p][i]*(b1-b2);
                    }
                }
                temp1=temp1*u[j]/(Sum_W[i]*Sum_W[i]+M_EPS);
                temp2=temp2+temp1;
            }
            B[i][t]=bt+temp2;
            if (toreturn==1 && fabs(B[i][t])>=(fabs(B[i][t])+1))
                toreturn=0;
        }
        for (q=0;q<n;q++) // To obtain A
        {
            at=0;
            for (l=0;l<M[i];l++)
            {
                R=S.read_Rule(l,i);
                at=at+W[l][i]*R->read_TSK(q+1);
            }
            at=at/(Sum_W[i]+M_EPS); // at[q][i]
            temp2=0;
            for (k=0;k<=n;k++)
            {
                temp1=0;
                for (l=0;l<M[i];l++)
                {
                    a1=S.read_Rule(l,i)->read_TSK(k);
                    for (p=0;p<M[i];p++)
                    {
                        a2=S.read_Rule(p,i)->read_TSK(k);
                        temp1=temp1+der_wx[l][i][q]*W[p][i]*(a1-a2);
                    }
                }
                temp1=temp1*X[k]/(Sum_W[i]*Sum_W[i]+M_EPS);
                temp2=temp2+temp1;
            }
            for (j=0;j<m;j++)
            {
                temp1=0;
                for (l=0;l<M[i];l++)
                {
                    b1=S.read_Rule(l,i)->read_TSK(j+n+1);
                    for (p=0;p<M[i];p++)
                    {
                        b2=S.read_Rule(p,i)->read_TSK(j+n+1);
                        temp1=temp1+der_wx[l][i][q]*W[p][i]*(b1-b2);
                    }
                }
                temp1=temp1*u[j]/(Sum_W[i]*Sum_W[i]+M_EPS);
                temp2=temp2+temp1;
            }
            A[i][q]=at+temp2;
            if (toreturn==1 && fabs(A[i][q])>=(fabs(A[i][q])+1))
                toreturn=0;
        }
    }
    // To obtain f(x0,u0)
    if (F!=NULL)
        if (Evaluate(x,F,S)!=1)
            toreturn=0;

    // Free the dynamic memory
    delete [] M;
    for (l=0;l<M_max;l++)
    {
        delete [] W[l];
        for (i=0;i<n;i++)
        {
            delete [] der_wu[l][i];
            delete [] der_wx[l][i];
        }
        delete [] der_wu[l];
        delete [] der_wx[l];
    }
    delete [] W;
    delete [] der_wx;
    delete [] der_wu;
    delete [] Sum_W;
    delete [] X;

    return toreturn;
}


// linearaprox ################################################################
int linearaprox(System &S, const double * const x, const double * const u, double **A, double **B, double h, double *F)
{
	int i,j,k;
	double *xu, *xu2, *dx, *dx2;

    const int n=S.outputs();
	const int m=S.inputs()-n;

	xu=new double[n+m];
	xu2=new double[n+m];

	for (i=0;i<n;i++)
	{
		xu[i]=x[i];
		xu2[i]=x[i];
	}
	for (i=0;i<m;i++)
	{
		xu[n+i]=u[i];
		xu2[n+i]=u[i];
	}
	dx=new double[n];

	if (Evaluate(xu,dx,S)!=1)
	{
		delete []dx;
		delete []xu2;
		delete []xu;
		return 0;
	}
	dx2=new double[n];

	for (j=0;j<n;j++)
	{
		xu2[j]=xu2[j]+h;

		if (Evaluate(xu2,dx2,S)!=1)
		{
			delete []dx;
			delete []xu2;
			delete []xu;
			delete []dx2;
			return 0;
		}
		for (i=0;i<n;i++)
			A[i][j]=(dx2[i]-dx[i])/h;

		xu2[j]=xu[j];
	}
	for (j=0;j<m;j++)
	{
	    int indice=n+j;
		xu2[indice]=xu2[indice]+h;
		if (Evaluate(xu2,dx2,S)!=1)
		{
			delete []dx;
			delete []xu2;
			delete []xu;
			delete []dx2;
			return 0;
		}
		for (i=0;i<n;i++)
			B[i][j]=(dx2[i]-dx[i])/h;
		xu2[indice]=xu[indice];
	}
	// To obtain f(x0,u0)
    if (F!=NULL)
        if (Evaluate(xu,F,S)!=1)
		{
			delete []dx;
			delete []xu2;
			delete []xu;
			delete []dx2;
            return 0;
		}
	delete []dx;
	delete []xu2;
	delete []xu;
	delete []dx2;
	return 1;
}

// printSystem #############################################################

int printSystem(const char *file,const System &S, char *inputs[], char *outputs[], int accuracy)
{
    int l,i,j,p,in,out;
    double value;
    FILE *Fs=fopen(file,"wt");
    if (!Fs)
        return 0;
    Rule R;
    Membership *P;
    char type[MAX_CHAR_LONG], temp[MAX_CHAR_LONG];
    char **InName,**OutName; // Names
    in=S.inputs();
    out=S.outputs();
    int *N=new int[out];
    for (j=0;j<out;j++)
        N[j]=S.rules(j);
    InName=new char *[in];
    OutName=new char *[out];

    for (i=0;i<in;i++)
    {
        InName[i]=new char [MAX_CHAR_LONG];
        if (inputs==NULL)
            sprintf(InName[i],"%s%d",U_FISInput,i+1);
        else sprintf(InName[i],"%s",inputs[i]);
    }
    for (i=0;i<out;i++)
    {
        OutName[i]=new char [MAX_CHAR_LONG];
        if (outputs==NULL)
            sprintf(OutName[i],"%s%d",U_FISOutput,i+1);
        else sprintf(OutName[i],"%s",outputs[i]);
    }
    for (j=0;j<out;j++)
    {
        for (l=0;l<N[j];l++)
        {
            if (!S.read_Rule(R,l,j))
            {
                fclose(Fs);
                printf("%s\n",E_RulesExceded);
                for (i=0;i<in;i++)
                    delete [] InName[i];
                delete [] InName;
                for (i=0;i<out;i++)
                    delete [] OutName[i];
                delete [] OutName;
                delete [] N;
                return 0;
            }
            fprintf(Fs,"%s ",U_If);
            for (i=0;i<in;i++)
            {
                P=R.read_function(i);
                fprintf(Fs,"%s %s %s(",InName[i],U_Is,MF_NAMES[(int)P->type()]);
                sprintf(temp,"%%.%dg;",accuracy);
                for (p=0;p<P->n_param();p++)
                    fprintf(Fs,temp,P->read(p));
                fseek(Fs,-1,SEEK_CUR);
                fprintf(Fs,") %s ",U_And);
            }
            fseek(Fs,-5,1);
            fprintf(Fs,"\n   %s %s = ",U_Then,OutName[j]);
            value=R.read_TSK(0);
            sprintf(temp,"%%.%dg",accuracy);
            if (value) fprintf(Fs,temp,value);
            for (i=0;i<in;i++)
            {
                value=R.read_TSK(i+1);
                sprintf(temp,"%%+.%dg*%%s",accuracy);
                if (value!=0)
                    fprintf(Fs,temp,value,InName[i]);
            }
            fprintf(Fs,"\n");
        }
    }
    fclose(Fs);
    delete [] N;
    for (i=0;i<in;i++)
        delete [] InName[i];
    delete [] InName;
    for (i=0;i<out;i++)
        delete [] OutName[i];
    delete [] OutName;
    return 1;
}

// NumbersOfAntecedentsData #####################################################

int NumbersOfAntecedentsData(System &S)
{
    int i,j,r,n,m,N,size_data;
    n=S.inputs();
    m=S.outputs();
    Rule *R;
    size_data=0;
    for (j=0;j<m;j++)
    {
        N=S.rules(j);
        for (r=0;r<N;r++)
        {
            R=S.read_Rule(r,j);
            for (i=0;i<n;i++)
                size_data=size_data+R->read_function(i)->n_param();
        }
    }
    return size_data;
}

int NumbersOfAntecedentsData(System &S, int out, int rule)
{
    int i,n,m,size_data=0;
    n=S.inputs();
    m=S.outputs();
    Rule *R;
    if (out>=m || rule>=S.rules(out))
        return 0;
    R=S.read_Rule(rule,out);
    for (i=0;i<n;i++)
        size_data=size_data+R->read_function(i)->n_param();
    return size_data;
}

// NumbersOfConsequentsData #####################################################

int NumbersOfConsequentsData(System &S)
{
    int j,n,m,size_data;
    n=S.inputs();
    m=S.outputs();
    size_data=0;
    for (j=0;j<m;j++)
        size_data=size_data+S.rules(j)*(n+1);
    return size_data;
}

int NumbersOfConsequentsData(System &S, int out, int rule)
{
    if (out>=S.outputs() || rule>=S.rules(out))
        return 0;
    return (S.inputs()+1);
}

// getAntecedents #########################################################

int getAntecedents(System &S, double *parameters)
{
    int i,j,r,p,n,m,N,n_param,size_data;
    n=S.inputs();
    m=S.outputs();
    n_param=NumbersOfAntecedentsData(S);

    int indice=0;
    Rule *R;
    Membership *P;
    for (j=0;j<m;j++)
    {
        N=S.rules(j);
        for (r=0;r<N;r++)
        {
            R=S.read_Rule(r,j);
            for (i=0;i<n;i++)
            {
                P=R->read_function(i);
                if (P->test()==0)
                    return 0;
                size_data=P->n_param();
                for (p=0;p<size_data;p++)
                {
                    parameters[indice]=P->read(p);
                    indice++;
                }
            }
        }
    }
    return n_param;
}

int getAntecedents(System &S, double *parameters, int out, int rule)
{
    int i,p,n,m,n_param,size_data;
    n=S.inputs();
    m=S.outputs();
    if (out>=m)
        return 0;
    if (rule>=S.rules(out))
        return 0;
    n_param=NumbersOfAntecedentsData(S,out,rule);

    int indice=0;
    Rule *R=S.read_Rule(rule,out);
    Membership *P;
    for (i=0;i<n;i++)
    {
        P=R->read_function(i);
        if (P->test()==0)
            return 0;
        size_data=P->n_param();
        for (p=0;p<size_data;p++)
        {
            parameters[indice]=P->read(p);
            indice++;
        }
    }
    return n_param;
}

// getConsequents #########################################################

int getConsequents(System &S, double *parameters)
{
    int indice=0,i,j,r,n,m,N,n_param,size_data;
    n=S.inputs();
    m=S.outputs();
    n_param=NumbersOfConsequentsData(S);

    Rule *R;
    for (j=0;j<m;j++)
    {
        N=S.rules(j);
        for (r=0;r<N;r++)
        {
            R=S.read_Rule(r,j);
            for (i=0;i<=n;i++)
            {
                parameters[indice]=R->read_TSK(i);
                indice++;
            }
        }
    }
    return n_param;
}

int getConsequents(System &S, double *parameters, int out, int rule)
{
    int i,n,m,n_param,size_data;
    n=S.inputs();
    m=S.outputs();
    if (out>=m || rule>=S.rules(out))
        return 0;
    n_param=NumbersOfConsequentsData(S,out,rule);

    Rule *R=S.read_Rule(rule,out);
    for (i=0;i<=n;i++)
        parameters[i]=R->read_TSK(i);
    return n_param;
}

// setAntecedents #######################################################

int setAntecedents(System &S, const double * const parameters)
{
    int i,j,r,p,n,m,N,n_param,size_data;
    n=S.inputs();
    m=S.outputs();
    if (S.test()==0)
        return 0;
    n_param=NumbersOfAntecedentsData(S);

    Membership *P;
    Rule *R;
    int indice=0;
    for (j=0;j<m;j++)
    {
        N=S.rules(j);
        for (r=0;r<N;r++)
        {
            R=S.read_Rule(r,j);
            for (i=0;i<n;i++)
            {
                P=R->read_function(i);
                size_data=P->n_param();
                for (p=0;p<size_data;p++)
                {
                    P->edit(p,parameters[indice]);
                    indice++;
                }
                if (P->test()==0)
                    return 0;
            }
        }
    }
    return 1;
}

int setAntecedents(System &S, const double * const parameters, int out, int rule)
{
    int i,p,n,m,n_param,size_data;
    n=S.inputs();
    m=S.outputs();
    if (S.test()==0 || out>=m || rule>=S.rules(out))
        return 0;
    n_param=NumbersOfAntecedentsData(S,out,rule);

    Membership *P;
    Rule *R=S.read_Rule(rule,out);;
    int indice=0;
    for (i=0;i<n;i++)
    {
        P=R->read_function(i);
        size_data=P->n_param();
        for (p=0;p<size_data;p++)
        {
            P->edit(p,parameters[indice]);
            indice++;
        }
        if (P->test()==0)
            return 0;
    }
    return 1;
}

// setConsequents #######################################################

int setConsequents(System &S, const double * const parameters)
{
    int i,j,r,p,n,m,N,n_param,size_data;
    n=S.inputs();
    m=S.outputs();
    if (S.test()==0)
        return 0;
    n_param=NumbersOfConsequentsData(S);

    Rule *R;
    int indice=0;
    for (j=0;j<m;j++)
    {
        N=S.rules(j);
        for (r=0;r<N;r++)
        {
            R=S.read_Rule(r,j);
            for (i=0;i<=n;i++)
            {
                R->write_TSK(parameters[indice],i);
                indice++;
            }
        }
    }
    return 1;
}

int setConsequents(System &S, const double * const parameters, int out, int rule)
{
    int i,p,n,m,n_param,size_data;
    n=S.inputs();
    m=S.outputs();
    if (S.test()==0)
        return 0;
    if (out>=m)
        return 0;
    if (rule>=S.rules(out))
        return 0;
    n_param=NumbersOfConsequentsData(S,out,rule);

    Rule *R=S.read_Rule(rule,out);
    for (i=0;i<=n;i++)
        R->write_TSK(parameters[i],i);
    return 1;
}

int activation(System &S, int output, int rule, double *point, double* W)
{
    int k,N,n;
    Rule *R;
	Membership *P;
    n=S.inputs();
    N=S.rules(output);
    if (output>=S.outputs() || rule>=S.rules(output))
        return 1;
    R=S.read_Rule(rule,output);
    *W=1;
    for (k=0;k<n;k++)
	{
		P=R->read_function(k);
		if (P->type()!=NOMF) // NOMF doesn't affect to the system
			(*W)=(*W)*P->eval(point[k]);
	}
    return 0;
}

int activation(System &S, int output, int rule, double *point, double* W, double *dW)
{
    int k,q,N,n;
    Membership *P;
    Rule *R;
    n=S.inputs();
    N=S.rules(output);
    if (output>=S.outputs() || rule>=S.rules(output))
        return 1;
    R=S.read_Rule(rule,output);
    *W=1;
    for (k=0;k<n;k++)
	{
		P=R->read_function(k);
		if (P->type()!=NOMF) // NOMF doesn't affect to the system
			(*W)=(*W)*P->eval(point[k]);
	}
    for (q=0;q<n;q++)
    {
        R=S.read_Rule(rule,output);
		P=R->read_function(q);
		if (P->type()!=NOMF) // NOMF doesn't affect to the system
		{
			dW[q]=P->evalder(point[q]);
			for (k=0;k<n;k++)
			{
				if (k!=q)
				{
					P=R->read_function(k);
					if (P->type()!=NOMF) // NOMF doesn't affect to the system
						dW[q]*=P->eval(point[k]);
				}
			}
		}
		else
			dW[q]=0;
    }
    return 0;
}

int subSystem(System &S,System &ParteS, int nrules, int *outputs, int *rules)
{
    int i,out;
    int n=S.inputs();
    int m=S.outputs();
    Rule *R;
    int *N=new int[m];
    for (i=0;i<m;i++)
        N[i]=0;
    for (i=0;i<nrules;i++)
        N[outputs[i]]++;
    for (i=0;i<m;i++)
    {
        if (N[i]==0)
        {
            delete []N;
            return 1;
        }
    }
    System Snuevo(n,m,N);
    for (i=0;i<m;i++)
        N[i]=0;
    for (i=0;i<nrules;i++)
    {
        out=*outputs;
        R=S.read_Rule(*rules,out);
        if(!R)
        {
            delete []N;
            return 1;
        }
        if(!Snuevo.write_Rule(*R,N[out],out))
        {
            delete []N;
            return 1;
        }
        N[out]++;
        outputs++;
        rules++;
    }
    ParteS=Snuevo;
    delete []N;
    return 0;
}
