/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../base/fuzzy.h"
#include "aux_matlab.h"
#include "mex.h"

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    int i, output, rule,m;
    double *Point,*W,*dW;
    
    if(nrhs!=4)
        ERRORMSG(E_NumberArgIn)
    if (nlhs>2)
        ERRORMSG(E_NumberArgOut)
    for (i=0;i<nrhs;i++)
    {
        if (mxIsEmpty(prhs[i]) || mxIsNaN(*mxGetPr(prhs[i])))
            ERRORMSG(E_ArgNoValid)
    }
    System S;
    if ( (mxGetN(prhs[1])!=1) || (mxGetM(prhs[1])!=1))
        ERRORMSG(E_Out_No_Scalar)
    if ( (mxGetN(prhs[2])!=1) || (mxGetM(prhs[2])!=1))
        ERRORMSG(E_R_No_Scalar)
    if (!readModel(prhs[0],S))
        ERRORMSG(E_Model)
    if (1!=mxGetN(prhs[3]))
        ERRORMSG(E_Column)
    else if (S.inputs()!=mxGetM(prhs[3]))
        ERRORMSG(E_PointCoherent)
    output=((int)*mxGetPr(prhs[1]))-1;
    rule=((int)*mxGetPr(prhs[2]))-1;
    if (output>=S.outputs())
        ERRORMSG(E_OutputExceded)
    else if (rule>=S.rules(output))
        ERRORMSG(E_RulesExceded)
    Point=mxGetPr(prhs[3]);
    plhs[0]=mxCreateScalarDouble(1);
    W=mxGetPr(plhs[0]);
    if (!plhs[0])
        ERRORMSG(E_NumberArgOut)
    if (nlhs==1)
    {
        if(activation(S,output,rule,Point,W))
        {
            mxDestroyArray(plhs[0]);
            ERRORMSG(E_NumberArgIn)
        }
    }
    else
    {
        m=S.inputs();
        plhs[1]=mxCreateDoubleMatrix(1,m,mxREAL);
        dW=mxGetPr(plhs[1]);
        if(activation(S,output,rule,Point,W,dW))
        {
            mxDestroyArray(plhs[0]);
            mxDestroyArray(plhs[1]);
            ERRORMSG(E_NumberArgIn)
        }
    }
}
