% FUZJAC Calculates the Jacobian matrix of the closed loop fuzzy system at a given point.
%
%       J=fuzjac(X, Plant, Controller)
%
% Arguments:
%
%   X -> State vector (column vector).
%
%   Plant -> Fuzzy model of Plant.
%
%   Controlador -> Fuzzy model of Controller.
%
%   J -> Jacobian matrix of closed loop system.
%
%                                    (  df1          df1  )
%       dx1                          | -----   ...  ----- |
%      ----- = f1(x)                 |  dx1          dxn  |
%       dt                           |                    |
%       ...                      J = |  ...    ...   ...  |
%       dxn                          |                    |
%      ----- = fn(x)                 |  dfn          dfn  |
%       dt                           | -----   ...  ----- |
%                                    (  dx1          dxn  )
%      x = [ x1 ; x2 ; ... ; xn]
%
% Fuzzy models could be a '.txt' or '.fis' file, or a 'FIS' variable from MATLAB
% Workspace. (See 'Fuzzy Logic Toolbox' help for more information about FIS).
%
%  See also FIS2TXT, FUZCOMB, FUZEVAL, FUZEQ, FUZLINEAR, FUZPRINT, TXT2FIS
%
% ##########################################################################
%
% Authors:
%    Antonio Javier Barragn Pia
%    URL: http://www.uhu.es/antonio.barragan
%    e-mail: antonio.barragan@diesia.uhu.es
%    
%    Jos Manuel Andjar Mrquez
%    e-mail: andujar@diesia.uhu.es
%
% Department of Electronics Engineering, Computer Systems and Automatics
% EPS La Rbida, University of Huelva (Spain)
%
% ##########################################################################
